@@ -17,6 +17,12 @@
 #include "strutils.h"
 #include "bitops.h"
 
+static int STRTOXX_EXIT_CODE = EXIT_FAILURE;
+
+void strutils_set_exitcode(int ex) {
+	STRTOXX_EXIT_CODE = ex;
+}
+
 static int do_scale_by_power (uintmax_t *x, int base, int power)
 {
 	while (power--) {
@@ -56,12 +62,13 @@ static int do_scale_by_power (uintmax_t *x, int base, int power)
  */
 int parse_size(const char *str, uintmax_t *res, int *power)
 {
-	char *p;
+	const char *p;
+	char *end;
 	uintmax_t x, frac = 0;
 	int base = 1024, rc = 0, pwr = 0, frac_zeros = 0;
 
-	static const char *suf  = "KMGTPEYZ";
-	static const char *suf2 = "kmgtpeyz";
+	static const char *suf  = "KMGTPEZY";
+	static const char *suf2 = "kmgtpezy";
 	const char *sp;
 
 	*res = 0;
@@ -77,25 +84,25 @@ int parse_size(const char *str, uintmax_t *res, int *power)
 	 * use lconv->negative_sign. But coreutils use the same solution,
 	 * so it's probably good enough...
 	 */
-	p = (char *) str;
+	p = str;
 	while (isspace((unsigned char) *p))
 		p++;
 	if (*p == '-') {
 		rc = -EINVAL;
 		goto err;
 	}
-	p = NULL;
 
-	errno = 0;
-	x = strtoumax(str, &p, 0);
+	errno = 0, end = NULL;
+	x = strtoumax(str, &end, 0);
 
-	if (p == str ||
+	if (end == str ||
 	    (errno != 0 && (x == UINTMAX_MAX || x == 0))) {
 		rc = errno ? -errno : -EINVAL;
 		goto err;
 	}
-	if (!p || !*p)
+	if (!end || !*end)
 		goto done;			/* without suffix */
+	p = end;
 
 	/*
 	 * Check size suffixes
@@ -107,25 +114,31 @@ check_suffix:
 		base = 1000;			/* XB, 10^N */
 	else if (*(p + 1)) {
 		struct lconv const *l = localeconv();
-		char *dp = l ? l->decimal_point : NULL;
+		const char *dp = l ? l->decimal_point : NULL;
 		size_t dpsz = dp ? strlen(dp) : 0;
 
 		if (frac == 0 && *p && dp && strncmp(dp, p, dpsz) == 0) {
-			char *fstr = p + dpsz;
+			const char *fstr = p + dpsz;
 
-			for (p = fstr; *p && *p == '0'; p++)
+			for (p = fstr; *p == '0'; p++)
 				frac_zeros++;
-			errno = 0, p = NULL;
-			frac = strtoumax(fstr, &p, 0);
-			if (p == fstr ||
-			    (errno != 0 && (frac == UINTMAX_MAX || frac == 0))) {
-				rc = errno ? -errno : -EINVAL;
-				goto err;
-			}
-			if (frac && (!p  || !*p)) {
+			fstr = p;
+			if (isdigit(*fstr)) {
+				errno = 0, end = NULL;
+				frac = strtoumax(fstr, &end, 0);
+				if (end == fstr ||
+				    (errno != 0 && (frac == UINTMAX_MAX || frac == 0))) {
+					rc = errno ? -errno : -EINVAL;
+					goto err;
+				}
+			} else
+				end = (char *) p;
+
+			if (frac && (!end  || !*end)) {
 				rc = -EINVAL;
 				goto err;		/* without suffix, but with frac */
 			}
+			p = end;
 			goto check_suffix;
 		}
 		rc = -EINVAL;
@@ -149,17 +162,39 @@ check_suffix:
 	if (power)
 		*power = pwr;
 	if (frac && pwr) {
-		int zeros_in_pwr = frac_zeros % 3;
-		int frac_pwr = pwr - (frac_zeros / 3) - 1;
-		uintmax_t y = frac * (zeros_in_pwr == 0 ? 100 :
-				      zeros_in_pwr == 1 ?  10 : 1);
+		int i;
+		uintmax_t frac_div = 10, frac_poz = 1, frac_base = 1;
 
-		if (frac_pwr < 0) {
-			rc = -EINVAL;
-			goto err;
-		}
-		do_scale_by_power(&y, base, frac_pwr);
-		x += y;
+		/* mega, giga, ... */
+		do_scale_by_power(&frac_base, base, pwr);
+
+		/* maximal divisor for last digit (e.g. for 0.05 is
+		 * frac_div=100, for 0.054 is frac_div=1000, etc.)
+		 */
+		while (frac_div < frac)
+			frac_div *= 10;
+
+		/* 'frac' is without zeros (5 means 0.5 as well as 0.05) */
+		for (i = 0; i < frac_zeros; i++)
+			frac_div *= 10;
+
+		/*
+		 * Go backwardly from last digit and add to result what the
+		 * digit represents in the frac_base. For example 0.25G
+		 *
+		 *  5 means 1GiB / (100/5)
+		 *  2 means 1GiB / (10/2)
+		 */
+		do {
+			unsigned int seg = frac % 10;		 /* last digit of the frac */
+			uintmax_t seg_div = frac_div / frac_poz; /* what represents the segment 1000, 100, .. */
+
+			frac /= 10;	/* remove last digit from frac */
+			frac_poz *= 10;
+
+			if (seg)
+				x += frac_base / (seg_div / seg);
+		} while (frac);
 	}
 done:
 	*res = x;
@@ -237,11 +272,11 @@ void *mempcpy(void *restrict dest, const void *restrict src, size_t n)
 #ifndef HAVE_STRNLEN
 size_t strnlen(const char *s, size_t maxlen)
 {
-        int i;
+        size_t i;
 
         for (i = 0; i < maxlen; i++) {
                 if (s[i] == '\0')
-                        return i + 1;
+                        return i;
         }
         return maxlen;
 }
@@ -269,6 +304,9 @@ char *strndup(const char *s, size_t n)
 }
 #endif
 
+static uint32_t _strtou32_or_err(const char *str, const char *errmesg, int base);
+static uint64_t _strtou64_or_err(const char *str, const char *errmesg, int base);
+
 int16_t strtos16_or_err(const char *str, const char *errmesg)
 {
 	int32_t num = strtos32_or_err(str, errmesg);
@@ -280,9 +318,9 @@ int16_t strtos16_or_err(const char *str, const char *errmesg)
 	return num;
 }
 
-uint16_t strtou16_or_err(const char *str, const char *errmesg)
+static uint16_t _strtou16_or_err(const char *str, const char *errmesg, int base)
 {
-	uint32_t num = strtou32_or_err(str, errmesg);
+	uint32_t num = _strtou32_or_err(str, errmesg, base);
 
 	if (num > UINT16_MAX) {
 		errno = ERANGE;
@@ -291,6 +329,16 @@ uint16_t strtou16_or_err(const char *str, const char *errmesg)
 	return num;
 }
 
+uint16_t strtou16_or_err(const char *str, const char *errmesg)
+{
+	return _strtou16_or_err(str, errmesg, 10);
+}
+
+uint16_t strtox16_or_err(const char *str, const char *errmesg)
+{
+	return _strtou16_or_err(str, errmesg, 16);
+}
+
 int32_t strtos32_or_err(const char *str, const char *errmesg)
 {
 	int64_t num = strtos64_or_err(str, errmesg);
@@ -302,9 +350,9 @@ int32_t strtos32_or_err(const char *str, const char *errmesg)
 	return num;
 }
 
-uint32_t strtou32_or_err(const char *str, const char *errmesg)
+static uint32_t _strtou32_or_err(const char *str, const char *errmesg, int base)
 {
-	uint64_t num = strtou64_or_err(str, errmesg);
+	uint64_t num = _strtou64_or_err(str, errmesg, base);
 
 	if (num > UINT32_MAX) {
 		errno = ERANGE;
@@ -313,6 +361,16 @@ uint32_t strtou32_or_err(const char *str, const char *errmesg)
 	return num;
 }
 
+uint32_t strtou32_or_err(const char *str, const char *errmesg)
+{
+	return _strtou32_or_err(str, errmesg, 10);
+}
+
+uint32_t strtox32_or_err(const char *str, const char *errmesg)
+{
+	return _strtou32_or_err(str, errmesg, 16);
+}
+
 int64_t strtos64_or_err(const char *str, const char *errmesg)
 {
 	int64_t num;
@@ -334,7 +392,7 @@ err:
 	errx(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
 }
 
-uint64_t strtou64_or_err(const char *str, const char *errmesg)
+static uint64_t _strtou64_or_err(const char *str, const char *errmesg, int base)
 {
 	uintmax_t num;
 	char *end = NULL;
@@ -342,7 +400,7 @@ uint64_t strtou64_or_err(const char *str, const char *errmesg)
 	errno = 0;
 	if (str == NULL || *str == '\0')
 		goto err;
-	num = strtoumax(str, &end, 10);
+	num = strtoumax(str, &end, base);
 
 	if (errno || str == end || (end && *end))
 		goto err;
@@ -355,6 +413,15 @@ err:
 	errx(STRTOXX_EXIT_CODE, "%s: '%s'", errmesg, str);
 }
 
+uint64_t strtou64_or_err(const char *str, const char *errmesg)
+{
+	return _strtou64_or_err(str, errmesg, 10);
+}
+
+uint64_t strtox64_or_err(const char *str, const char *errmesg)
+{
+	return _strtou64_or_err(str, errmesg, 16);
+}
 
 double strtod_or_err(const char *str, const char *errmesg)
 {
@@ -446,7 +513,7 @@ void strtotimeval_or_err(const char *str, struct timeval *tv, const char *errmes
  * Converts stat->st_mode to ls(1)-like mode string. The size of "str" must
  * be 11 bytes.
  */
-void xstrmode(mode_t mode, char *str)
+char *xstrmode(mode_t mode, char *str)
 {
 	unsigned short i = 0;
 
@@ -481,10 +548,12 @@ void xstrmode(mode_t mode, char *str)
 		? (mode & S_IXOTH ? 't' : 'T')
 		: (mode & S_IXOTH ? 'x' : '-'));
 	str[i] = '\0';
+
+	return str;
 }
 
 /*
- * returns exponent (2^x=n) in range KiB..PiB
+ * returns exponent (2^x=n) in range KiB..EiB (2^10..2^60)
  */
 static int get_exp(uint64_t n)
 {
@@ -509,6 +578,7 @@ char *size_to_human_string(int options, uint64_t bytes)
 	if (options & SIZE_SUFFIX_SPACE)
 		*psuf++ = ' ';
 
+
 	exp  = get_exp(bytes);
 	c    = *(letters + (exp ? exp / 10 : 0));
 	dec  = exp ? bytes / (1ULL << exp) : bytes;
@@ -527,11 +597,17 @@ char *size_to_human_string(int options, uint64_t bytes)
 	 *                 exp, suffix[0], dec, frac);
 	 */
 
+	/* round */
 	if (frac) {
-		/* round */
-		frac = (frac / (1ULL << (exp - 10)) + 50) / 100;
-		if (frac == 10)
-			dec++, frac = 0;
+		if (options & SIZE_DECIMAL_2DIGITS) {
+			frac = (frac / (1ULL << (exp - 10)) + 5) / 10;
+			if (frac % 10 == 0)
+				frac /= 10;	/* convert N.90 to N.9 */
+		} else {
+			frac = (frac / (1ULL << (exp - 10)) + 50) / 100;
+			if (frac == 10)
+				dec++, frac = 0;
+		}
 	}
 
 	if (frac) {
@@ -793,8 +869,10 @@ int streq_paths(const char *a, const char *b)
 		    ((a_seg && *a_seg == '/') || (b_seg && *b_seg == '/')))
 			return 1;
 
+		if (!a_seg || !b_seg)
+			break;
 		if (a_sz != b_sz || strncmp(a_seg, b_seg, a_sz) != 0)
-			return 0;
+			break;
 
 		a = a_seg + a_sz;
 		b = b_seg + b_sz;
@@ -932,7 +1010,7 @@ int skip_fline(FILE *fp)
 	} while (1);
 }
 
-#ifdef TEST_PROGRAM
+#ifdef TEST_PROGRAM_STRUTILS
 
 static int test_strutils_sizes(int argc, char *argv[])
 {
@@ -984,4 +1062,4 @@ int main(int argc, char *argv[])
 
 	return EXIT_FAILURE;
 }
-#endif /* TEST_PROGRAM */
+#endif /* TEST_PROGRAM_STRUTILS */
