@@ -39,7 +39,6 @@
 #include "canonicalize.h"		/* $(top_srcdir)/include */
 #include "pathnames.h"
 #include "sysfs.h"
-#include "at.h"
 
 /*
  * Find a dev struct in the cache by device name, if available.
@@ -51,28 +50,55 @@ blkid_dev blkid_get_dev(blkid_cache cache, const char *devname, int flags)
 {
 	blkid_dev dev = NULL, tmp;
 	struct list_head *p, *pnext;
+	char *cn = NULL;
 
 	if (!cache || !devname)
 		return NULL;
 
+	/* search by name */
 	list_for_each(p, &cache->bic_devs) {
 		tmp = list_entry(p, struct blkid_struct_dev, bid_devs);
 		if (strcmp(tmp->bid_name, devname))
 			continue;
-
-		DBG(DEVNAME, ul_debug("found devname %s in cache", tmp->bid_name));
 		dev = tmp;
 		break;
 	}
 
+	/* try canonicalize the name */
+	if (!dev && (cn = canonicalize_path(devname))) {
+		if (strcmp(cn, devname) != 0) {
+			DBG(DEVNAME, ul_debug("search canonical %s", cn));
+			list_for_each(p, &cache->bic_devs) {
+				tmp = list_entry(p, struct blkid_struct_dev, bid_devs);
+				if (strcmp(tmp->bid_name, cn))
+					continue;
+				dev = tmp;
+
+				/* update name returned by blkid_dev_devname() */
+				free(dev->bid_xname);
+				dev->bid_xname = strdup(devname);
+				break;
+			}
+		} else {
+			free(cn);
+			cn = NULL;
+		}
+	}
+
 	if (!dev && (flags & BLKID_DEV_CREATE)) {
 		if (access(devname, F_OK) < 0)
-			return NULL;
+			goto done;
 		dev = blkid_new_dev();
 		if (!dev)
-			return NULL;
+			goto done;
 		dev->bid_time = INT_MIN;
-		dev->bid_name = strdup(devname);
+		if (cn) {
+			dev->bid_name = cn;
+			dev->bid_xname = strdup(devname);
+			cn = NULL;	/* see free() below */
+		} else
+			dev->bid_name = strdup(devname);
+
 		dev->bid_cache = cache;
 		list_add_tail(&dev->bid_devs, &cache->bic_devs);
 		cache->bic_flags |= BLKID_BIC_FL_CHANGED;
@@ -81,7 +107,7 @@ blkid_dev blkid_get_dev(blkid_cache cache, const char *devname, int flags)
 	if (flags & BLKID_DEV_VERIFY) {
 		dev = blkid_verify(cache, dev);
 		if (!dev || !(dev->bid_flags & BLKID_BID_FL_VERIFIED))
-			return dev;
+			goto done;
 		/*
 		 * If the device is verified, then search the blkid
 		 * cache for any entries that match on the type, uuid,
@@ -112,6 +138,10 @@ blkid_dev blkid_get_dev(blkid_cache cache, const char *devname, int flags)
 				blkid_free_dev(dev2);
 		}
 	}
+done:
+	if (dev)
+		DBG(DEVNAME, ul_debug("%s requested, found %s in cache", devname, dev->bid_name));
+	free(cn);
 	return dev;
 }
 
@@ -122,7 +152,7 @@ static int is_dm_leaf(const char *devname)
 {
 	struct dirent	*de, *d_de;
 	DIR		*dir, *d_dir;
-	char		path[256];
+	char		path[NAME_MAX + 18 + 1];
 	int		ret = 1;
 
 	if ((dir = opendir("/sys/block")) == NULL)
@@ -171,7 +201,7 @@ static void probe_one(blkid_cache cache, const char *ptname,
 			dev = blkid_verify(cache, tmp);
 			if (dev && (dev->bid_flags & BLKID_BID_FL_VERIFIED))
 				break;
-			dev = 0;
+			dev = NULL;
 		}
 	}
 	if (dev && dev->bid_devno == devno)
@@ -183,7 +213,7 @@ static void probe_one(blkid_cache cache, const char *ptname,
 	if (!strncmp(ptname, "dm-", 3) && isdigit(ptname[3])) {
 		devname = canonicalize_dm_name(ptname);
 		if (!devname)
-			blkid__scan_dir("/dev/mapper", devno, 0, &devname);
+			blkid__scan_dir("/dev/mapper", devno, NULL, &devname);
 		if (devname)
 			goto get_dev;
 	}
@@ -213,7 +243,7 @@ static void probe_one(blkid_cache cache, const char *ptname,
 	}
 	/* Do a short-cut scan of /dev/mapper first */
 	if (!devname)
-		blkid__scan_dir("/dev/mapper", devno, 0, &devname);
+		blkid__scan_dir("/dev/mapper", devno, NULL, &devname);
 	if (!devname) {
 		devname = blkid_devno_to_devname(devno);
 		if (!devname)
@@ -397,7 +427,7 @@ ubi_probe_all(blkid_cache cache, int only_if_new)
 				continue;
 			if (!strcmp(name, "ubi_ctrl"))
 				continue;
-			if (fstat_at(dirfd(dir), *dirname, name, &st, 0))
+			if (fstatat(dirfd(dir), name, &st, 0))
 				continue;
 
 			dev = st.st_rdev;
@@ -419,9 +449,10 @@ static int probe_all(blkid_cache cache, int only_if_new)
 {
 	FILE *proc;
 	char line[1024];
-	char ptname0[128 + 1], ptname1[128 + 1], *ptname = 0;
+	char ptname0[128 + 1], ptname1[128 + 1], *ptname = NULL;
 	char *ptnames[2];
-	dev_t devs[2];
+	dev_t devs[2] = { 0, 0 };
+	int iswhole[2] = { 0, 0 };
 	int ma, mi;
 	unsigned long long sz;
 	int lens[2] = { 0, 0 };
@@ -435,7 +466,7 @@ static int probe_all(blkid_cache cache, int only_if_new)
 		return -BLKID_ERR_PARAM;
 
 	if (cache->bic_flags & BLKID_BIC_FL_PROBED &&
-	    time(0) - cache->bic_time < BLKID_PROBE_INTERVAL)
+	    time(NULL) - cache->bic_time < BLKID_PROBE_INTERVAL)
 		return 0;
 
 	blkid_read_cache(cache);
@@ -459,7 +490,7 @@ static int probe_all(blkid_cache cache, int only_if_new)
 			continue;
 		devs[which] = makedev(ma, mi);
 
-		DBG(DEVNAME, ul_debug("read partition name %s", ptname));
+		DBG(DEVNAME, ul_debug("read device name %s", ptname));
 
 		/* Skip whole disk devs unless they have no partitions.
 		 * If base name of device has changed, also
@@ -469,15 +500,14 @@ static int probe_all(blkid_cache cache, int only_if_new)
 		 *
 		 * Skip extended partitions.
 		 * heuristic: size is 1
-		 *
-		 * FIXME: skip /dev/{ida,cciss,rd} whole-disk devs
 		 */
 
 		lens[which] = strlen(ptname);
+		iswhole[which] = sysfs_devno_is_wholedisk(devs[which]);
 
-		/* ends in a digit, clearly a partition, so check */
-		if (isdigit(ptname[lens[which] - 1])) {
-			DBG(DEVNAME, ul_debug("partition dev %s, devno 0x%04X",
+		/* probably partition, so check */
+		if (!iswhole[which]) {
+			DBG(DEVNAME, ul_debug(" partition dev %s, devno 0x%04X",
 				   ptname, (unsigned int) devs[which]));
 
 			if (sz > 1)
@@ -491,7 +521,9 @@ static int probe_all(blkid_cache cache, int only_if_new)
 		 * on it, remove the whole-disk dev from the cache if
 		 * it exists.
 		 */
-		if (lens[last] && !strncmp(ptnames[last], ptname, lens[last])) {
+		if (lens[last] && iswhole[last]
+		    && !strncmp(ptnames[last], ptname, lens[last])) {
+
 			list_for_each_safe(p, pnext, &cache->bic_devs) {
 				blkid_dev tmp;
 
@@ -499,14 +531,14 @@ static int probe_all(blkid_cache cache, int only_if_new)
 				tmp = list_entry(p, struct blkid_struct_dev,
 						 bid_devs);
 				if (tmp->bid_devno == devs[last]) {
-					DBG(DEVNAME, ul_debug("freeing %s",
+					DBG(DEVNAME, ul_debug(" freeing %s",
 						       tmp->bid_name));
 					blkid_free_dev(tmp);
 					cache->bic_flags |= BLKID_BIC_FL_CHANGED;
 					break;
 				}
 			}
-			lens[last] = 0;
+			lens[last] = 0;		/* mark as checked */
 		}
 		/*
 		 * If last was not checked because it looked like a whole-disk
@@ -514,11 +546,12 @@ static int probe_all(blkid_cache cache, int only_if_new)
 		 * check last as well.
 		 */
 		if (lens[last] && strncmp(ptnames[last], ptname, lens[last])) {
-			DBG(DEVNAME, ul_debug("whole dev %s, devno 0x%04X",
+			DBG(DEVNAME, ul_debug(" whole dev %s, devno 0x%04X",
 				   ptnames[last], (unsigned int) devs[last]));
 			probe_one(cache, ptnames[last], devs[last], 0,
 				  only_if_new, 0);
-			lens[last] = 0;
+
+			lens[last] = 0;		/* mark as checked */
 		}
 	}
 
@@ -535,6 +568,7 @@ static int probe_all(blkid_cache cache, int only_if_new)
  */
 static int probe_all_removable(blkid_cache cache)
 {
+	struct path_cxt *pc;
 	DIR *dir;
 	struct dirent *d;
 
@@ -545,8 +579,9 @@ static int probe_all_removable(blkid_cache cache)
 	if (!dir)
 		return -BLKID_ERR_PROC;
 
+	pc = ul_new_path(NULL);
+
 	while((d = readdir(dir))) {
-		struct sysfs_cxt sysfs = UL_SYSFSCXT_EMPTY;
 		int removable = 0;
 		dev_t devno;
 
@@ -559,20 +594,19 @@ static int probe_all_removable(blkid_cache cache)
 		     ((d->d_name[1] == '.') && (d->d_name[2] == 0))))
 			continue;
 
-		devno = sysfs_devname_to_devno(d->d_name, NULL);
+		devno = sysfs_devname_to_devno(d->d_name);
 		if (!devno)
 			continue;
 
-		if (sysfs_init(&sysfs, devno, NULL) == 0) {
-			if (sysfs_read_int(&sysfs, "removable", &removable) != 0)
+		if (sysfs_blkdev_init_path(pc, devno, NULL) == 0
+		    && ul_path_read_s32(pc, &removable, "removable") != 0)
 				removable = 0;
-			sysfs_deinit(&sysfs);
-		}
 
 		if (removable)
 			probe_one(cache, d->d_name, devno, 0, 0, 1);
 	}
 
+	ul_unref_path(pc);
 	closedir(dir);
 	return 0;
 }
@@ -593,7 +627,7 @@ int blkid_probe_all(blkid_cache cache)
 	DBG(PROBE, ul_debug("Begin blkid_probe_all()"));
 	ret = probe_all(cache, 0);
 	if (ret == 0) {
-		cache->bic_time = time(0);
+		cache->bic_time = time(NULL);
 		cache->bic_flags |= BLKID_BIC_FL_PROBED;
 	}
 	DBG(PROBE, ul_debug("End blkid_probe_all() [rc=%d]", ret));
